from .bootstrap3 import Bootstrap3Renderer
from .fontawesome4 import FontAwesome4Renderer
from .icon import IconRenderer
from .image import ImageRenderer
from .material import MaterialRenderer

__all__ = ["IconRenderer", "Bootstrap3Renderer", "FontAwesome4Renderer", "MaterialRenderer", "ImageRenderer"]



from django.conf import settings
from django.test import TestCase, override_settings

from .test_template_tags import render_template


class ImageTest(TestCase):
    """Test the Image Renderer."""

    def test_icons(self):
        self.assertHTMLEqual(
            render_template('{% icon "icons8-icons8-48" renderer="ImageRenderer" %}'),
            '<img src="/static/icons/icons8-icons8-48.png" alt="Icon of Icons8 Icons8 48" class="icon icon-icons8-icons8-48">',  # noqa
        )
        self.assertHTMLEqual(
            render_template('{% icon "icons8-48" renderer="ImageRenderer" %}'),
            '<img src="/static/icons/icons8-48.png" alt="Icon of Icons8 48" class="icon icon-icons8-48">',  # noqa
        )
        self.assertHTMLEqual(
            render_template('{% icon "icons8-s:48" renderer="ImageRenderer" %}'),
            '<img src="/static/icons/icons8-48.png" alt="Icon of Icons8" class="icon icon-size-48 icon-icons8">',
        )
        self.assertHTMLEqual(
            render_template('{% icon "icons8-c:b" renderer="ImageRenderer" %}'),
            '<img src="/static/icons/icons8-b.png" alt="Icon of Icons8" class="icon icon-color-b icon-icons8">',
        )
        self.assertHTMLEqual(
            render_template('{% icon "icons8-c:b-s:48" renderer="ImageRenderer" %}'),
            '<img src="/static/icons/icons8-b-48.png" alt="Icon of Icons8" class="icon icon-color-b icon-size-48 icon-icons8">',  # noqa
        )
        self.assertHTMLEqual(
            render_template('{% icon "icons8-s:48-c:b" renderer="ImageRenderer" %}'),
            '<img src="/static/icons/icons8-b-48.png" alt="Icon of Icons8" class="icon icon-color-b icon-size-48 icon-icons8">',  # noqa
        )
        self.assertHTMLEqual(
            render_template('{% icon "icons8-s:48-c:b" renderer="ImageRenderer" format="jpg" %}'),
            '<img src="/static/icons/icons8-b-48.jpg" alt="Icon of Icons8" class="icon icon-color-b icon-size-48 icon-icons8">',  # noqa
        )
        self.assertHTMLEqual(
            render_template('{% icon "icons8" renderer="ImageRenderer" size="48" color="b" %}'),
            '<img src="/static/icons/icons8-b-48.png" alt="Icon of Icons8" class="icon icon-color-b icon-size-48 icon-icons8">',  # noqa
        )
        self.assertHTMLEqual(
            render_template('{% icon "icons8-s:48-c:b" renderer="ImageRenderer" size="48" color="b" %}'),
            '<img src="/static/icons/icons8-s:48-c:b-b-48.png" alt="Icon of Icons8 S:48 C:B" class="icon icon-color-b icon-size-48 icon-icons8-s:48-c:b">',  # noqa
        )
        self.assertHTMLEqual(
            render_template('{% icon "icons8-s:48-c:b" renderer="ImageRenderer" size="48" color="b" format="jpg" %}'),
            '<img src="/static/icons/icons8-s:48-c:b-b-48.jpg" alt="Icon of Icons8 S:48 C:B" class="icon icon-color-b icon-size-48 icon-icons8-s:48-c:b">',  # noqa
        )

    def test_path_dev(self):
        self.assertHTMLEqual(
            render_template('{% icon "icons8-48" renderer="ImageRenderer" %}'),
            '<img src="'
            + settings.STATIC_URL
            + 'icons/icons8-48.png" alt="Icon of Icons8 48" class="icon icon-icons8-48">',
        )
        self.assertHTMLEqual(
            render_template('{% icon "icons8-48" renderer="ImageRenderer" %}'),
            '<img src="/static/icons/icons8-48.png" alt="Icon of Icons8 48" class="icon icon-icons8-48">',
        )

    @override_settings(DEBUG=False, STATIC_URL="http://static.example.com/")
    def test_path_deploy(self):
        self.assertHTMLEqual(
            render_template('{% icon "icons8-48" renderer="ImageRenderer" %}'),
            '<img src="'
            + settings.STATIC_URL
            + 'icons/icons8-48.png" alt="Icon of Icons8 48" class="icon icon-icons8-48">',
        )
        self.assertHTMLEqual(
            render_template('{% icon "icons8-48" renderer="ImageRenderer" %}'),
            '<img src="http://static.example.com/icons/icons8-48.png" alt="Icon of Icons8 48" class="icon icon-icons8-48">',  # noqa
        )

    def test_custom_renderer_image_root(self):
        DJANGO_ICONS = {
            "DEFAULTS": {"renderer": "fontawesome4", "attrs": {"aria-hidden": True}},
            "RENDERERS": {"image": "ImageRenderer", "hd-image": "tests.app.renderers.CustomImageRenderer"},
            "ICONS": {"edit": {"renderer": "image"}, "feather": {"renderer": "hd-image"}},
        }
        with override_settings(DJANGO_ICONS=DJANGO_ICONS):
            self.assertHTMLEqual(
                render_template("{% icon 'edit' %}"),
                '<img src="/static/icons/edit.png" alt="Icon of Edit" class="icon icon-edit">',
            )
            self.assertHTMLEqual(
                render_template("{% icon 'feather' %}"),
                '<img src="/static/hd-icons/feather.png" alt="Icon of Feather" class="icon icon-feather">',
            )

    def test_custom_renderer_image_prefix(self):
        DJANGO_ICONS = {
            "DEFAULTS": {"renderer": "fontawesome4", "attrs": {"aria-hidden": True}},
            "RENDERERS": {"icons8": "tests.app.renderers.CustomIcons8Renderer"},
            "ICONS": {"icons8-48": {"renderer": "icons8"}},
        }
        with override_settings(DJANGO_ICONS=DJANGO_ICONS):
            self.assertHTMLEqual(
                render_template("{% icon 'icons8-48' %}"),
                '<img src="/static/icons/icons8-icons8-48.png" alt="Icon of Icons8 48" class="icon icon-icons8-48">',
            )



from django.test import TestCase

from django_icons.css import merge_css_list, merge_css_text


class CSSTest(TestCase):
    """Test the CSS utilities."""

    def test_merge_css_list(self):
        # No arguments results in an empty list
        self.assertEqual(merge_css_list(), [])
        # Falsy arguments are ignores
        self.assertEqual(merge_css_list(None, False, ""), [])
        # Spaces don't matter without any other results
        self.assertEqual(merge_css_list("    "), [])
        # Spaces don't matter with other classes
        self.assertEqual(merge_css_list("  foo  "), ["foo"])
        # Spaces don't matter with other classes
        self.assertEqual(merge_css_list("  foo  ", "bar"), ["foo", "bar"])
        # Space separated classes are split
        self.assertEqual(merge_css_list("foo bar"), ["foo", "bar"])
        # Nested lists are fine
        self.assertEqual(merge_css_list(["foo", "bar"], "end"), ["foo", "bar", "end"])
        # Duplicates are removed and first time class is encountered determines order
        self.assertEqual(merge_css_list(["foo", "bar"], "foo"), ["foo", "bar"])
        # Tabs and newlines are valid whitespace
        self.assertEqual(merge_css_list("foo\nbar\tend"), ["foo", "bar", "end"])
        # Let's go crazy
        self.assertEqual(
            merge_css_list(
                "foo\nbar\tend",
                ["foo", ["bar", "bar", ["foo bar end"]]],
                "\n\t foo Foo",
                [[], [], [["Foo-Bar-End"]]],
                ("end", "bar", "foo", "Foo"),
            ),
            ["foo", "bar", "end", "Foo", "Foo-Bar-End"],
        )
        self.assertEqual(
            merge_css_text(
                "foo\nbar\tend",
                ["foo", ["bar", "bar", ["foo bar end"]]],
                "\n\t foo Foo",
                [[], [], [["Foo-Bar-End"]]],
                ("end", "bar", "foo", "Foo"),
            ),
            "foo bar end Foo Foo-Bar-End",
        )



from django.test import TestCase

from django_icons.renderers import Bootstrap3Renderer, FontAwesome4Renderer, IconRenderer, ImageRenderer
from django_icons.utils import _get_setting, get_icon_kwargs, get_icon_kwargs_from_settings, get_icon_renderer


class UtilsTest(TestCase):
    """Test the utility functions."""

    def test_get_setting(self):
        with self.settings(DJANGO_ICONS=None):
            self.assertEqual(_get_setting("SECTION", "name"), None)
            self.assertEqual(_get_setting("SECTION", "name", "foo"), "foo")
        with self.settings(DJANGO_ICONS={"SECTION": {"name": "bar"}}):
            self.assertEqual(_get_setting("SECTION", "name"), "bar")
            self.assertEqual(_get_setting("SECTION", "name", "foo"), "bar")

    def test_get_icon_kwargs_from_settings(self):
        with self.settings(DJANGO_ICONS=None):
            self.assertEqual(get_icon_kwargs_from_settings("info"), {"name": "info"})
        with self.settings(DJANGO_ICONS={"ICONS": {"info": "info-sign"}}):
            self.assertEqual(get_icon_kwargs_from_settings("info"), {"name": "info-sign"})
        with self.settings(DJANGO_ICONS={"ICONS": {"info": {"name": "info-sign"}}}):
            self.assertEqual(get_icon_kwargs_from_settings("info"), {"name": "info-sign"})
        with self.settings(DJANGO_ICONS={"ICONS": {"info": {"title": "Information"}}}):
            self.assertEqual(
                get_icon_kwargs_from_settings("info"),
                {"name": "info", "title": "Information"},
            )

    def test_get_icon_kwargs(self):
        with self.settings(DJANGO_ICONS=None):
            self.assertEqual(
                get_icon_kwargs("info", "size-lg color-red"),
                {"name": "info", "extra_classes": ["size-lg", "color-red"]},
            )
        with self.settings(DJANGO_ICONS={"ICONS": {"info": "info-sign"}}):
            self.assertEqual(get_icon_kwargs("info"), {"name": "info-sign"})
        with self.settings(DJANGO_ICONS={"ICONS": {"info": {"name": "info-sign"}}}):
            self.assertEqual(get_icon_kwargs("info"), {"name": "info-sign"})
        with self.settings(DJANGO_ICONS={"ICONS": {"info": {"title": "Information"}}}):
            self.assertEqual(get_icon_kwargs("info"), {"name": "info", "title": "Information"})

    def test_get_icon_renderer(self):
        with self.settings(DJANGO_ICONS=None):
            self.assertEqual(get_icon_renderer(), IconRenderer)
            self.assertEqual(get_icon_renderer("icon"), IconRenderer)
            self.assertEqual(get_icon_renderer("fontawesome4"), FontAwesome4Renderer)
            self.assertEqual(get_icon_renderer("bootstrap3"), Bootstrap3Renderer)
            self.assertEqual(get_icon_renderer("image"), ImageRenderer)
            self.assertEqual(
                get_icon_renderer("django_icons.renderers.Bootstrap3Renderer"),
                Bootstrap3Renderer,
            )
            self.assertEqual(
                get_icon_renderer("Bootstrap3Renderer"),
                Bootstrap3Renderer,
            )



from django_icons.renderers.icon import IconRenderer


class Bootstrap3Renderer(IconRenderer):
    """Render a Bootstrap 3 icon."""

    def get_class(self):
        return f"glyphicon glyphicon-{self.name}"



from django_icons.renderers import IconRenderer, ImageRenderer


class CustomSvgRenderer(IconRenderer):
    """
    Render an SVG.

    <svg class="svg-icon" viewBox="0 0 32 32"><use xlink:href="#icon-feather"></use></svg>
    """

    def get_tag(self):
        return "svg"

    def get_class(self):
        return "svg-icon"

    def get_attrs(self):
        attrs = super(CustomSvgRenderer, self).get_attrs()
        attrs["viewBox"] = "0 0 32 32"
        return attrs

    def get_content(self):
        return '<use xlink:href="#icon-{name}"></use>'.format(name=self.name)


class CustomImageRenderer(ImageRenderer):
    def get_image_root(cls):
        return "hd-icons"


class CustomIcons8Renderer(ImageRenderer):
    def get_image_prefix(self):
        return "icons8-"



from django.template import Context, Template
from django.test import TestCase


def render_template(content, **context_args):
    """Create a template with content ``content`` that first loads the font-awesome tags."""
    template = Template("{% load icons %}" + content)
    return template.render(Context(context_args))


class TemplateTagsTest(TestCase):
    """Test all template tags."""

    def test_load_icons(self):
        # Loading the template should not generate an error
        self.assertEqual(render_template(""), "")

    def test_default_renderer(self):
        self.assertEqual(render_template('{% icon "name" %}'), '<i class="name"></i>')



from django.test import TestCase

from .test_template_tags import render_template


class BaseTest(TestCase):
    """Test the IconRenderer."""

    def test_icons(self):
        self.assertEqual(
            render_template('{% icon "" %}'),
            "<i></i>",
        )
        self.assertEqual(
            render_template('{% icon "user" %}'),
            '<i class="user"></i>',
        )
        self.assertEqual(
            render_template('{% icon "fas fa-user fa-2x" %}'),
            '<i class="fas fa-user fa-2x"></i>',
        )

    def test_extra_classes(self):
        self.assertEqual(
            render_template('{% icon "extra-triangle" %}'),
            '<i class="fas fa-triangle fa-fw extra"></i>',
        )
        self.assertEqual(
            render_template('{% icon "extra-triangle" extra_classes="and more" %}'),
            '<i class="fas fa-triangle fa-fw extra and more"></i>',
        )
        self.assertEqual(
            render_template('{% icon "extra-triangle" "and" extra_classes="more" %}'),
            '<i class="fas fa-triangle fa-fw extra and more"></i>',
        )
